<?php

/**
 * SHD SM-VIEW Feed Jira API
 *
 * @package    i-doit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class feed_api_model_jira extends feed_api_model_abstract
{

    /**
     * Name of this module.
     *
     * @var string
     */
    protected $module_name = 'jira';

    /**
     * Name of this category.
     *
     * @var string
     */
    protected $category_name = 'C__CATG__SMVIEW_FEED_JIRA';

    /**
     * Constructor.
     */
    function __construct()
    {
        parent::__construct();
        $this->dao = isys_cmdb_dao_smview_feed_jira::instance(isys_application::instance()->database);
    }

    /**
     * Read objects.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'filter' => array|null,
     * )
     *
     * @return array
     */
    public function read($p_params)
    {
        $result = parent::read($p_params);
        return $this->_remove_password_field($result, isset($p_params["id"]));
    }

    /**
     * Create configuration.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'address'  => string,
     *     'name'     => string,
     *     'username' => string,
     *     'password' => string,
     * )
     *
     * @return array
     */
    public function create($p_params)
    {
        $required_parameters = [
            'address',
            'name',
        ];
        $optional_parameters = [
            'username',
            'password',
        ];
        $data = $p_params['data'];
        return $this->_create($data, $required_parameters, $optional_parameters);
    }

    /**
     * Update configuration.
     *
     * @param  array $p_params
     *
     * $p_param structure:
     * array (
     *     'id'       => int,
     *     'address'  => string,
     *     'name'     => string,
     *     'username' => string,
     *     'password' => string,
     * )
     * @return array
     */
    public function update($p_params)
    {
        $parameters = array (
            'address',
            'name',
            'username',
            'password',
        );

        $id = $p_params['id'];
        $data = $p_params['data'];
        return $this->_update($data, $id, $parameters);
    }

    /**
     * Delete configuration.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'id' => int,
     * )
     *
     * @return array
     */
    public function delete($p_params)
    {
        return $this->_delete('id', $p_params);
    }
}
