<?php

/**
 * SHD SM-VIEW Feed Jira Filter API
 *
 * @package    i-doit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class feed_api_model_jira_filter extends feed_api_model_abstract
{

    /**
     * Name of this module.
     *
     * @var string
     */
    protected $module_name = 'jira_filter';

    /**
     * Name of this category.
     *
     * @var string
     */
    protected $category_name = 'C__CATG__SMVIEW_FEED_JIRA';

    /**
     * Constructor.
     */
    function __construct()
    {
        parent::__construct();
        $this->dao = isys_cmdb_dao_smview_feed_jira_filter::instance(isys_application::instance()->database);
    }

    /**
     * Create configuration.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'obj_type_id' => int,
     *     'field_id'    => int,
     *     'field_name'  => string,
     *     'condition'   => string,
     *     'value'       => string,
     * )
     *
     * @return array
     */
    public function create($p_params)
    {
        $required_parameters = array (
            'obj_type_id',
            'field_id',
            'field_name',
            'condition',
            'value',
        );
        $data = $p_params['data'];
        return $this->_create($data, $required_parameters);
    }

    /**
     * Update configuration.
     *
     * @param  array $p_params
     *
     * $p_param structure:
     * array (
     *     'id'          => int,
     *     'obj_type_id' => int,
     *     'field_id'    => int,
     *     'field_name'  => string,
     *     'condition'   => string,
     *     'value'       => string,
     * )
     * @return array
     */
    public function update($p_params)
    {
        $parameters = array (
            'obj_type_id',
            'field_id',
            'field_name',
            'condition',
            'value',
        );

        $id = $p_params['id'];
        $data = $p_params['data'];
        return $this->_update($data, $id, $parameters);
    }

    /**
     * Delete configuration.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'id' => int,
     * )
     *
     * @return array
     */
    public function delete($p_params)
    {
        return $this->_delete('id', $p_params);
    }
}
