import * as React from 'react'

import { FieldProps, FilterProps } from './MapperCompose'

interface FilterRowProps {
    field_id: string,
    image_url: string,
    language: {[key: string]: string},
    fields: FieldProps[],
    filters: FilterProps[],
    conditions: {[key: string]: string},
    onAdd: Function,
    onConditionChange: Function,
    onValueChange: Function,
    onDelete: Function,
}

export class FilterRow extends React.Component<FilterRowProps, {}> {
  constructor (props: FilterRowProps) {
    super(props)

    this.handleAdd = this.handleAdd.bind(this)
    this.handleConditionChange = this.handleConditionChange.bind(this)
    this.handleValueChange = this.handleValueChange.bind(this)
    this.handleDelete = this.handleDelete.bind(this)
    this.handleDeleteAll = this.handleDeleteAll.bind(this)
  }

  handleAdd () {
    this.props.onAdd(this.props.field_id)
  }

  handleConditionChange (event: React.ChangeEvent<HTMLSelectElement>) {
    const li = event.currentTarget.closest('li')
    this.props.onConditionChange(li.id, event.currentTarget.value)
  }

  handleValueChange (event: React.ChangeEvent<HTMLInputElement>) {
    const li = event.currentTarget.closest('li')
    this.props.onValueChange(li.id, event.currentTarget.value)
  }

  handleDelete (event: React.MouseEvent<HTMLButtonElement>) {
    const li = event.currentTarget.closest('li')
    this.props.onDelete(li.id)
  }

  handleDeleteAll () {
    this.props.onDelete(null, this.props.field_id)
  }

  render () {
    let title = ''
    for (const key in this.props.fields) {
      if (!this.props.fields.hasOwnProperty(key)) {
        continue
      }
      const field = this.props.fields[key]
      if (field.id == this.props.field_id) {
        title = field.name
      }
    }

    const options = []
    for (const key in this.props.conditions) {
      if (this.props.conditions.hasOwnProperty(key)) {
        const label = this.props.conditions[key]
        options.push(<option key={this.props.field_id + '_' + key}
          value={key}>{label}</option>)
      }
    }

    const add_img = this.props.image_url + 'add.png'
    const delete_img = this.props.image_url + 'cross.png'
    const lis = []
    for (const key in this.props.filters) {
      if (!this.props.filters.hasOwnProperty(key)) {
        continue
      }

      const filter = this.props.filters[key]
      const field_name = 'target_filter_' + filter.id
      lis.push(<li id={filter.id.toString()} key={'filter_' + filter.id}>
        <div className="fl p5">
          <input type="hidden" name={field_name} value={filter.field_id}/>
          <select id={'filter_condition_' + filter.id}
            name={'filter_condition_' + filter.id}
            defaultValue={filter.condition}
            className='input'
            onChange={this.handleConditionChange}>{options}</select>
        </div>
        <div className="fl p5">
          <input type="text" className="input" name={'filter_value_' + filter.id}
            value={filter.value} onChange={this.handleValueChange}/>
        </div>
        <div className="fl p5">
          <button type="button" className="btn btn-block" onClick={this.handleDelete}>
            <img src={delete_img} alt="+" className="mr5" />
          </button>
        </div>
      </li>)
    }

    return <div className="border mt10 category-container">
      <h4 className="gradient p5 mouse-pointer">
        <button type="button" className="btn btn-small close fr" onClick={this.handleDeleteAll} title={this.props.language.LC__CMDB__CATG__SMVIEW_FEED__DELETE_ASSIGNMENT}>
          <img src={delete_img} alt="x" />
        </button>
        <span>{title}</span>
      </h4>

      <div className="border-top filter-container">
        <div className="property-label">
          <strong className="text-shadow">{this.props.language.LC__CMDB__CATG__SMVIEW_FEED__FILTER}</strong>
        </div>
        <div className="fl">
          <ul>
            {lis}
          </ul>
        </div>
        <div className="fl" style={{ marginLeft: '20%' }}>
          <button type="button" className="mt5 btn btn-block fl m5" onClick={this.handleAdd}>
            <img src={add_img} alt="+" className="mr5" />
          </button>
        </div>
      </div>
    </div>
  }
}
