import * as React from 'react'

export interface SelectProps {
    id?: string,
    name: string,
    options: {[key: string]: string},
    selected_option?: string,
    onChange?: Function,
    disabled?: boolean,
}

export class Select extends React.Component<SelectProps, {}> {
  constructor (props: SelectProps) {
    super(props)
    this.handleChange = this.handleChange.bind(this)
  }

  handleChange (event: React.ChangeEvent<HTMLSelectElement>) {
    if (this.props.onChange) {
      this.props.onChange(event.currentTarget.value)
    }
  }

  render () {
    const options = []
    options.push(<option key={this.props.name + '_-1'} value='-1'>-</option>)
    for (const key in this.props.options) {
      const row = this.props.options[key]
      options.push(<option key={this.props.name + '_' + key} value={key}>{row}</option>)
    }
    return <select id={this.props.id}
      name={this.props.name}
      defaultValue={this.props.selected_option}
      className='input'
      onChange={this.handleChange}
      disabled={this.props.disabled}>{options}</select>
  }
}
