import * as React from 'react'

export interface SelectorComposeProps {
  object_types: ObjectType[],
  language: {[key: string]: string},
  is_editable: boolean,
}

interface SelectorComposeState {
  object_types: ObjectType[],
}

interface ObjectType {
  id: number,
  title: string,
  const: string,
  is_selected: boolean,
}

export class SelectorCompose extends React.Component<SelectorComposeProps, SelectorComposeState> {
  constructor (props: SelectorComposeProps) {
    super(props)
    this.state = {
      object_types: this.props.object_types
    }
    this.getTableBody = this.getTableBody.bind(this)
    this.handleObjectTypeSelection = this.handleObjectTypeSelection.bind(this)
  }

  getTableBody () {
    const trs: JSX.Element[] = []
    for (const key in this.state.object_types) {
      if (this.state.object_types.hasOwnProperty(key)) {
        const object_type: ObjectType = this.state.object_types[key]
        if (!this.props.is_editable && !object_type.is_selected) {
          continue
        }
        trs.push(<Row is_editable={this.props.is_editable} object_type={object_type} onObjectTypeSelected={this.handleObjectTypeSelection} />)
      }
    }
    return <tbody>{trs}</tbody>
  }

  handleObjectTypeSelection (object_type_const: string, is_selected: boolean) {
    const object_types = this.state.object_types.filter(o => {
      if (o.const == object_type_const) {
        o.is_selected = is_selected
      }
      return o
    })

    // write state
    this.setState((prevState) => ({
      ...prevState,
      object_types: object_types
    }), () => {
      const object_types = []
      for (const key in this.state.object_types) {
        if (this.state.object_types.hasOwnProperty(key)) {
          const object_type: ObjectType = this.state.object_types[key]
          if (object_type.is_selected) {
            object_types.push(object_type.const)
          }
        }
      }

      // directly save object type constants to UI element (to avoid handling in backend)
      const ots = object_types.join(';')
      const item = document.getElementById('C__CATG__SMVIEW_FEED_INDEXED_OBJECT_TYPES')
      item.setAttribute('value', ots)
    })
  }

  render () {
    return <table className="mainTable">
      <thead>
        <tr>
          <th>&nbsp;</th>
          <th>{this.props.language.LC_UNIVERSAL__OBJECT_TYPE}</th>
        </tr>
      </thead>
      {this.getTableBody()}
    </table>
  }
}

interface RowProps {
  object_type: ObjectType,
  is_editable: boolean,
  onObjectTypeSelected: Function,
}

class Row extends React.Component<RowProps, {}> {
  constructor (props: RowProps) {
    super(props)
    this.handleObjectTypeSelected = this.handleObjectTypeSelected.bind(this)
  }

  handleObjectTypeSelected (e: React.ChangeEvent<HTMLInputElement>) {
    const id = e.target.value
    this.props.onObjectTypeSelected(id, e.target.checked)
  }

  render () {
    return <tr key={this.props.object_type.id}>
      <td>
        <input type="checkbox"
          name="object_types[]"
          onChange={this.handleObjectTypeSelected}
          defaultChecked={this.props.object_type.is_selected}
          disabled={!this.props.is_editable}
          value={this.props.object_type.const} />
      </td>
      <td>{this.props.object_type.title}</td>
    </tr>
  }
}
