const { CleanWebpackPlugin } = require('clean-webpack-plugin');
const { resolve } = require('path');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');

module.exports = {
    entry: {
        jira_mapper:[
            './src/jira_mapper/index.tsx',
            './../sass/jira_mapper.scss'
        ],
        object_type_selector:[
            './src/object_type_selector/index.tsx',
            './../sass/object_type_selector.scss'
        ]
    },
    output: {
        filename: 'feed_[name].js',
        path: resolve(__dirname, './../dist/')
    },
    resolve: {
        cacheWithContext: false,
        extensions: ['.ts', '.tsx', '.js', '.json'],
        symlinks: false,
    },
    module: {
        rules: [
            {
                test: /\.tsx?$/,
                include: resolve(__dirname, 'src'),
                use: 'ts-loader',
                exclude: /node_modules/,
            },
            // sass to css file
            {
                test: /\.(scss)$/,
                include: resolve(__dirname, './../sass'),
                use: [MiniCssExtractPlugin.loader, 'css-loader', 'sass-loader'],
                sideEffects: true
            }
        ]
    },
    optimization: {
        minimize: false,
        // separate 3rd-party libs because they won't change so often
        moduleIds: 'hashed',
        chunkIds: 'named',
        splitChunks: {
            chunks: 'async',
            maxAsyncRequests: 6,
            maxInitialRequests: 4,
            cacheGroups: {
                vendors: {
                    test: /[\\/]node_modules[\\/]/,
                    name: 'vendors',
                    chunks: 'all',
                    filename: 'vendors.js'
                }
            }
        }
    },
    plugins: [
        new CleanWebpackPlugin(),
        new MiniCssExtractPlugin({
            filename: 'css/[name]-style.css'
        })
    ]
};
