<?php

use idoit\Module\Feed\ProxyApi\Jira;

/**
 * i-doit
 *
 * DAO: global category for SM-VIEW Feed Jira.
 *
 * @package    i-doit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_category_g_smview_feed_jira extends isys_cmdb_dao_category_g_smview_feed_abstract
{

    /**
     * Category's name. Will be used for the identifier, constant, main table, and many more.
     *
     * @var string
     */
    protected $m_category = 'smview_feed_jira';

    /**
     * Prefix of field names.
     *
     * @var string
     */
    protected $_field_name_prefix = 'C__CATG__SMVIEW_FEED__JIRA_';

    /**
     * Get all fields of this object.
     *
     * @return array
     * @throws Exception
     */
    protected function _getFields()
    {
        $language = isys_application::instance()->container->get('language');
        try {
            $proxy_api = new Jira();
            // @todo: determine instance(s) to be queried
            $fields = $proxy_api->get_all_fields();
        } catch (isys_exception_database $e) {
            isys_notify::error($language->get('LC__CMDB__CATG__SMVIEW_FEED__CACHE_CONFIGURATION_INVALID'));
            $fields = [];
        } catch (isys_exception_api $e) {
            isys_notify::error($language->get('LC__CMDB__CATG__SMVIEW_FEED__CACHE_BACKEND_UNREACHABLE'));
            $fields = [];
        }
        return $fields;
    }

    /**
     * Get issues from OrientDB, filtered by object ID.
     *
     * @param  int|null $p_obj_id
     * @return array
     * @throws Exception
     */
    protected function _getElements($p_obj_id = null)
    {
        if ($this->_cached_elements === null || ! isset($this->_cached_elements[$p_obj_id])) {
            $language = isys_application::instance()->container->get('language');
            $mapping_dao = new isys_cmdb_dao_smview_feed_jira_2_obj_type($this->m_db);
            $filter_dao = new isys_cmdb_dao_smview_feed_jira_filter($this->m_db);
            try {
                $conditions = ! empty($p_obj_id) ? $this->_get_conditions($p_obj_id, $mapping_dao, $filter_dao) : [];
            } catch (Exception $e) {
                isys_application::instance()->container->get('notify')->notice($e->getMessage());
                $this->_cached_elements[$p_obj_id] = [];
                return [];
            }

            if (empty($conditions)) {
                $this->_cached_elements[$p_obj_id] = [];
                return [];
            }

            try {
                $proxy_api = new Jira();
            } catch (isys_exception_database $e) {
                isys_notify::error($language->get('LC__CMDB__CATG__SMVIEW_FEED__CACHE_CONFIGURATION_INVALID'));
                return [];
            }

            try {
                // @todo: determine instance(s) to be queried
                $this->_cached_elements[$p_obj_id] = $proxy_api->get_all_issues($conditions);
            } catch (isys_exception_database $e) {
                isys_notify::error($language->get('LC__CMDB__CATG__SMVIEW_FEED__CACHE_CONFIGURATION_INVALID'));
                return [];
            } catch (isys_exception_api $e) {
                isys_notify::error($language->get('LC__CMDB__CATG__SMVIEW_FEED__CACHE_BACKEND_UNREACHABLE'));
                return [];
            }
        }
        return $this->_cached_elements[$p_obj_id];
    }

    /**
     * Map field ID from internal separator back to '.' notation.
     *
     * @param  string $field_name
     * @return string
     */
    protected function _get_field_id($field_name)
    {
        $field_name = str_replace(self::FIELD_NAME_SEPARATOR, '.', $field_name);
        return 'fields.' . $field_name;
    }
}
