<?php

use idoit\Module\Feed\ProxyApi\AbstractApi;

/**
 * i-doit
 *
 * Abstract API DAO for SM-VIEW Feed.
 *
 * @package    i-doit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2021 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
abstract class isys_cmdb_dao_smview_feed_abstract_api extends isys_cmdb_dao_smview_feed_abstract
{

    /**
     * Get api instance.
     *
     * @return AbstractApi
     */
    abstract protected function _getApiInstance();

    /**
     * Create a new object.
     *
     * @param array $params
     * @param array $optionalParams
     * @return int|false
     * @throws isys_exception_dao
     */
    public function create_object($params, $optionalParams = [])
    {
        $data = array_merge($params, $optionalParams);
        $data['url'] = $data['address'];
        try {
            $apiInstance = $this->_getApiInstance();
            $apiInstance->create_instance($data);
        } catch (isys_exception_database | isys_exception_api $e) {
            // ignore exception
        }

        return parent::create_object($params, $optionalParams);
    }

    /**
     * Update an existing object.
     *
     * @param int $id
     * @param array $params
     * @param array $optionalParams
     * @return boolean
     * @throws isys_exception_dao
     */
    public function update_object($id, $params, $optionalParams = [])
    {
        $data = array_merge($params, $optionalParams);
        if (isset($data['address'])) {
            $data['url'] = $data['address'];
        }

        if (isset($params['password'])) {
            // remove password, if not updated (contains some ****)
            if (substr_count($params['password'], '*') > 2) {
                unset($params['password']);
                unset($data['password']);
            }
        }

        $apiInstance = $this->_getApiInstance();
        $cacheInstance = null;
        try {
            $cacheInstance = $apiInstance->get_instance($id);
        } catch (isys_exception_database | isys_exception_api $e) {
            // ignore exception
        }

        try {
            if (empty($cacheInstance)) {
                $apiInstance->create_instance($data);
            } else {
                $apiInstance->update_instance($id, $data);
            }
        } catch (isys_exception_database | isys_exception_api $e) {
            // ignore exception
        }

        return parent::update_object($id, $params, $optionalParams);
    }

    /**
     * Delete an existing object.
     *
     * @param int $id
     * @return boolean
     * @throws isys_exception_dao
     */
    public function delete_object($id)
    {
        try {
            $apiInstance = $this->_getApiInstance();
            $apiInstance->delete_instance($id);
        } catch (isys_exception_database | isys_exception_api $e) {
            // ignore exception
        }

        return parent::delete_object($id);
    }
}
