<?php

/**
 * i-doit
 * DAO: list of Feed Virtual Device.
 *
 * @package    idoit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2020 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_cmdb_dao_list_catg_smview_feed_virtual_device extends isys_cmdb_dao_list
{

    /**
     * Complete data of virtual device.
     *
     * @var array
     */
    private $_virtual_device_data;

    /**
     * Return constant of category.
     *
     * @return integer
     */
    public function get_category()
    {
        return defined_or_default('C__CATG__SMVIEW_FEED_VIRTUAL_DEVICE');
    }

    /**
     * Return constant of category type.
     *
     * @return integer
     */
    public function get_category_type()
    {
        return C__CMDB__CATEGORY__TYPE_GLOBAL;
    }

    /**
     * Returns array with table headers.
     *
     * @return array
     * @throws Exception
     */
    public function get_fields()
    {
//        $dao = new isys_cmdb_dao_list_catg_connector(isys_application::instance()->database);
//        $fields = $dao->get_fields();

        // use fields from connector DAO and add virtual device properties
        $language = isys_application::instance()->container->get('language');
        return [
            'isys_catg_connector_list__title' => $language->get('LC__CMDB__CATG__TITLE'),
            'isys_connection_type__title'     => $language->get('LC__CATG__CONNECTOR__CONNECTION_TYPE'),
            'connector_name'                  => $language->get('LC__CMDB__CONNECTED_WITH'),
            'cable_object'                    => $language->get('LC__CMDB__OBJTYPE__CABLE'),
            'end_device_location'             => $language->get('LC__CMDB__CATG__SMVIEW_FEED__VIRTUAL_DEVICE_LOCATION_END_DEVICE'),
            'isys_virtual_device'             => $language->get('LC__CMDB__CATG__SMVIEW_FEED__VIRTUAL_DEVICE_HOST_NAME'),
            'isys_virtual_device_port'        => $language->get('LC__CMDB__CATG__SMVIEW_FEED__VIRTUAL_DEVICE_PORT'),
            'isys_virtual_device_last_seen'   => $language->get('LC__CMDB__CATG__SMVIEW_FEED__VIRTUAL_DEVICE_LAST_SEEN'),
        ];
    }

    /**
     * General get_result.
     *
     * @param  string  $p_strTableName
     * @param  int $p_object_id
     * @param  int $p_cRecStatus
     * @return isys_component_dao_result
     * @throws isys_exception_dao_cmdb
     */
    public function get_result($p_strTableName = null, $p_object_id, $p_cRecStatus = null)
    {
        // modified this method to filter only outputs
        if (is_object($this->m_cat_dao)) {
            if ($p_object_id <= 0) {
                $p_object_id = $_GET[C__CMDB__GET__OBJECT];
            }

            if (method_exists($this->m_cat_dao, "get_data")) {
                $p_cRecStatus = ((!empty($p_cRecStatus))) ? $p_cRecStatus : $this->get_rec_status();
                if (is_string($p_cRecStatus) && !is_numeric($p_cRecStatus)) {
                    $p_cRecStatus = constant($p_cRecStatus);
                }

                // modified SQL to filter only outputs
                $filter = ' AND isys_catg_connector_list.isys_catg_connector_list__type = ' . $this->m_cat_dao->convert_sql_int(C__CONNECTOR__OUTPUT);
                return $this->m_cat_dao->get_data(null, $p_object_id, $filter, null, $p_cRecStatus);
            } else {
                throw new isys_exception_dao_cmdb(get_class($this->m_cat_dao) . "::get_data not implemented, yet.");
            }
        } else {
            throw new isys_exception_dao_cmdb("isys_component_dao_object_table_list::\$m_cat_dao undefined.");
        }
    }

    /**
     * Modify row method will be called for each row to alter its content.
     *
     * @param array &$p_row
     */
    public function modify_row(&$p_row)
    {
        // set 'isys_connection_type__id' to generate a link in HTML list
        // important: ID must be of type int(10)
        $p_row['isys_connection_type__id'] = $p_row['isys_catg_connector_list__id'];

        // note: $this->m_cat_dao is changed to type isys_cmdb_dao_category_g_connector
        // in UI class isys_cmdb_ui_category_g_smview_feed_virtual_device -> process_list
        $dao = new isys_cmdb_dao_category_g_smview_feed_virtual_device($this->m_cat_dao->m_db);
        if (! $this->_virtual_device_data) {
            // load virtual device data only once
            $obj_id = $this->m_cat_dao->get_object_id();
            $this->_virtual_device_data = $dao->get_data_as_array(null, $obj_id);
        }

        if (! empty($p_row['isys_catg_connector_list__isys_catg_connector_list__id']) || ! empty($p_row['con_connector'])) {
            // trace port to end device
            try {
                $endDevice = $dao->tracePort($p_row);
                $p_row['end_device_location'] = $endDevice['location'];
            } catch (isys_exception_database $e) {
                // pass
            }
        }

        $ports = $dao->get_virtual_device_ports_by_connector($p_row['isys_catg_connector_list__title'], $this->_virtual_device_data);
        if ($ports && count($ports) > 0) {
            $port = $ports[0];
            $p_row['isys_virtual_device_port'] = $port['isys_port'];
            $p_row['isys_virtual_device'] = $port['isys_host_name'];
            $p_row['isys_virtual_device_last_seen'] = $port['isys_last_seen'];
        }
    }
}
