<?php

/**
 * i-doit
 *
 * UI: global category for SM-VIEW Feed Jira.
 *
 * @package    i-doit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_ui_category_g_smview_feed_jira extends isys_cmdb_ui_category_global
{

    /**
     * Overwriting this method to set the correct template.
     *
     * @param  isys_cmdb_dao_category $p_cat
     * @throws Exception
     */
    public function init(isys_cmdb_dao_category $p_cat)
    {
        parent::init($p_cat);
        $this->set_template(isys_module_smview_feed::get_tpl_dir() . 'module/cmdb/catg__smview_feed_jira.tpl');
    }

    /**
     * Processes category data list for multi-valued categories.
     *
     * @param  isys_cmdb_dao_category $p_cat
     * @param  array                  $p_get_param_override
     * @param  string                 $p_strVarName
     * @param  string                 $p_strTemplateName
     * @param  bool                   $p_bCheckbox
     * @param  bool                   $p_bOrderLink
     * @param  string                 $p_db_field_name
     *
     * @return null
     * @throws isys_exception_cmdb
     * @throws isys_exception_general
     */
    public function process_list(isys_cmdb_dao_category &$p_cat, $p_get_param_override = null, $p_strVarName = null, $p_strTemplateName = null, $p_bCheckbox = true, $p_bOrderLink = true, $p_db_field_name = null)
    {
        parent::process_list($p_cat, $p_get_param_override, $p_strVarName, $p_strTemplateName, $p_bCheckbox, $p_bOrderLink, $p_db_field_name);

        isys_component_template_navbar::getInstance()
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__NEW'))
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__ARCHIVE'))
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__QUICK_PURGE'))
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__PRINT'))
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__EXPORT_AS_CSV'));

        $lang = isys_application::instance()->container->get('language');
        isys_component_template_navbar::getInstance()
            ->append_button(
                $lang->get('LC__CMDB__CATG__SMVIEW_FEED__CREATE_TICKET_TITLE'),
                'C__NAVBAR__BUTTON__ANALYSIS', $this->_get_button_options());
        return null;
    }

    /**
     * Get options of relation button.
     *
     * @return array
     * @throws Exception
     */
    private function _get_button_options()
    {
        // inject javascript code
        $objId = $_GET[C__CMDB__GET__OBJECT];
        $lang = isys_application::instance()->container->get('language');
        return [
            'tooltip'       => $lang->get('LC__CMDB__CATG__SMVIEW_FEED__CREATE_TICKET_TITLE'),
            "icon"          => 'icons/silk/report_magnify.png',
            'icon_inactive' => 'icons/silk/report_magnify.png',
            "navmode"       => defined_or_default('C__NAVMODE__DUPLICATE'), // important to prevent JS syntax error
            "js_onclick"    => " onclick=\"get_popup('new_jira_ticket', null, '480', '280', {'func':'new_jira_ticket', 'obj_id':'$objId'});\""
        ];
    }

    /**
     * Processes the UI for this category
     *
     * @param  isys_cmdb_dao_category $p_cat The corresponding category DAO
     * @throws isys_exception_dao_cmdb
     */
    public function process(isys_cmdb_dao_category $p_cat)
    {
        parent::process($p_cat);
        $this->disable_ajax_save();
        $this->deactivate_commentary();

        $properties = $p_cat->get_properties();
        $issue = $p_cat->get_data($this->m_category_data_id, $this->m_object_id)->get_row();

        // replace API URL with browse-URL
        $pos = stripos($issue['isys_self'], 'rest/api/');
        $url = substr($issue['isys_self'], 0, $pos);
        $url .= 'browse/' . $issue['isys_key'];
        $issue['self'] = $url;

        $this->get_template_component()
            ->assign('properties', $properties)
            ->assign('issue', $issue);

        isys_component_template_navbar::getInstance()
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__EDIT'));
    }

    /**
     * Fetches category's title from database.
     *
     * @param  isys_cmdb_dao_category &$p_cat Category's DAO
     * @return string  Category's title, otherwise error message.
     * @throws Exception
     */
    public function gui_get_title(isys_cmdb_dao_category &$p_cat)
    {
        $language = isys_application::instance()->container->get('language');
        $l_cat_data = $p_cat->get_general_data();
        $description = $l_cat_data['key'];
        if (empty($description)) {
            return $language->get('LC__CMDB__CATG__SMVIEW_FEED__JIRA_TITLE');
        }
        return $language->get('LC__CMDB__CATG__SMVIEW_FEED__JIRA_ISSUE') . ": $description";
    }
}
