<?php

use idoit\Module\Feed\ProxyApi\Jira;
use idoit\Module\Feed\ProxyApi\Idoit;

/**
 * Feed config AJAX handler
 *
 * @package    i-doit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_ajax_handler_feed_config extends isys_ajax_handler
{

    /**
     * Init method, which gets called from the framework.
     */
    public function init()
    {
        // We set the header information because we don't accept anything than JSON.
        header('Content-Type: application/json');

        $l_return = [
            'success' => true,
            'message' => null,
            'data'    => null
        ];

        try {
            switch ($this->m_get['func']) {
                case 'reindex':
                    if (isys_auth_cmdb::instance()->is_allowed_to(isys_auth::EDIT, "CATEGORY/C__CATG__SMVIEW_FEED_JIRA")) {
                        $instance_id = ! empty($this->m_get['instance_id']) ? $this->m_get['instance_id'] : null;
                        try {
                            $proxy_api = new Jira();
                            $proxy_api->re_index_instance($instance_id);
                        } catch (isys_exception_api $e) {
                        } catch (Exception $e) {
                        }
                    }
                    break;
                case 'reindex-idoit':
                    $instance_id = ! empty($this->m_get['instance_id']) ? $this->m_get['instance_id'] : null;
                    try {
                        $proxy_api = new Idoit();
                        $proxy_api->re_index_instance($instance_id);
                    } catch (isys_exception_api $e) {
                    } catch (Exception $e) {
                    }
                    break;
                case 'new_jira_ticket':
                    $obj_id = ! empty($this->m_get['obj_id']) ? $this->m_get['obj_id'] : null;
                    $obj_title = ! empty($this->m_get['obj_title']) ? $this->m_get['obj_title'] : null;
                    $obj_description = ! empty($this->m_get['obj_description']) ? $this->m_get['obj_description'] : null;
                    $author = ! empty($this->m_get['author']) ? $this->m_get['author'] : null;

                    $jira = new \idoit\Module\Feed\JiraApi\Jira();
                    $jira->create_jira_ticket($obj_id, $obj_title, $obj_description, $author);
                    break;
            }
        }
        catch (isys_exception $e) {
            $l_return['success'] = false;
            $l_return['message'] = $e->getMessage();
        }

        echo isys_format_json::encode($l_return);

        $this->_die();
    }

    /**
     * This method defines, if the hypergate needs to be included for this request.
     *
     * @static
     * @return boolean
     */
    public static function needs_hypergate()
    {
        return true;
    }
}
