<?php

use idoit\Module\Feed\ProxyApi\Jira;
use idoit\Module\Feed\ProxyApi\VirtualDevice;
use idoit\Module\Feed\DependentObjectManager\AbstractManager;
use idoit\Module\Feed\DependentObjectManager\JiraManager;
use idoit\Module\Feed\DependentObjectManager\IdoitManager;
use idoit\Module\Feed\VirtualDeviceImporter;

/**
 * i-doit
 *
 * SM-VIEW Feed module.
 *
 * @package    idoit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2019 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_module_smview_feed extends isys_module
{
    const DISPLAY_IN_MAIN_MENU = true;
    const DISPLAY_IN_SYSTEM_MENU = false;

    /**
     * @var bool
     */
    protected static $m_licenced = true;

    /**
     * Variable which holds the database component.
     *
     * @var isys_component_database
     */
    protected $m_db = null;

    /**
     * Variable which holds the template component.
     *
     * @var isys_component_template
     */
    protected $m_tpl = null;

    /**
     * Variable which holds the template component.
     *
     * @var isys_component_tree
     */
    protected $m_tree = null;

    /**
     * Property mapping.
     *
     * @var array
     */
    protected $_cache_mapping = array(
        'address'  => 'C__SMVIEW_FEED__ADDRESS',
        'username' => 'C__SMVIEW_FEED__USERNAME',
        'password' => 'C__SMVIEW_FEED__PASSWORD',
    );

    /**
     * Static method for retrieving the path, to the modules templates.
     *
     * @static
     * @return string
     */
    public static function get_tpl_dir()
    {
        return __DIR__ . '/templates/';
    }

    /**
     * Represents the name, of the corresponding auth class - "false" means, there is none.
     *
     * @var string
     */
    const AUTH_CLASS_NAME = 'isys_auth_module_smview_feed';

    /**
     * Method for retrieving the path to the module directory (needed for includes).
     *
     * @static
     * @return string
     */
    public static function get_module_dir()
    {
        return __DIR__ . DS;
    }

    /**
     * Method for retrieving the www-path (needed for the frontend).
     *
     * @static
     * @return  string
     */
    public static function get_www_dir() {
        return isys_application::instance()->www_path . 'src/classes/modules/smview_feed/';
    }

    /**
     * Method for retrieving the www-path to the assets directory (needed for the frontend).
     *
     * @static
     * @return  string
     */
    public static function get_assets_www_dir() {
        return isys_application::instance()->www_path . 'src/classes/modules/smview_feed/assets/';
    }

    /**
     * Initializes the module.
     *
     * @param  isys_module_request $p_req
     * @return isys_module_smview_feed
     */
    public function init(isys_module_request $p_req)
    {
        $this->m_db = $p_req->get_database();
        $this->m_tpl = $p_req->get_template();
        $this->m_tree = $p_req->get_menutree();
        return $this;
    }

    /**
     * Start method.
     *
     * @desc   Starts module process
     * @throws isys_exception_database
     * @throws isys_exception_dao
     * @throws Exception
     */
    public function start()
    {
        $l_template = $this->m_tpl;
        $l_tree = $this->m_tree;

        # index
        $language = isys_application::instance()->container->get('language');
        $l_tree->add_node(0, -1, $language->get("LC__CMDB__SMVIEW_FEED__TREE_TITLE"),
            isys_helper_link::create_url([
                C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW_FEED'),
                C__GET__TREE_NODE => 0,
            ])
        );

        # JIRA adapter
        $l_tree->add_node(1, 0, $language->get("LC__CMDB__SMVIEW_FEED__ADAPTER_JIRA"),
            isys_helper_link::create_url([
                C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW_FEED'),
                C__GET__TREE_NODE => 1,
                'request' => 'adapter_jira',
            ]),
            '',
            self::get_www_dir() . 'theme/img/icons/jira.png'
        );

        // add a node for each assigned object type
        $cat_g_id = defined_or_default('C__CATG__SMVIEW_FEED_JIRA');
        $node_id = $this->_addAssignedObjectTypes($l_tree, $cat_g_id, 1, 2, 'object_type_mapping');

        # Idoit adapter
        $l_tree->add_node($node_id, 0, $language->get("LC__CMDB__SMVIEW_FEED__ADAPTER_IDOIT"),
            isys_helper_link::create_url([
                C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW_FEED'),
                C__GET__TREE_NODE => $node_id,
                'request'         => 'adapter_idoit',
            ]),
            '',
            self::get_www_dir() . 'theme/img/icons/i-doit.png'
        );
        $node_id++;

        # Virtual device adapter
        $l_tree->add_node($node_id, 0, $language->get("LC__CMDB__SMVIEW_FEED__ADAPTER_VIRTUAL_DEVICE"),
            isys_helper_link::create_url([
                C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW_FEED'),
                C__GET__TREE_NODE => $node_id,
                'request'         => 'adapter_virtual_device',
            ]),
            '',
            self::get_www_dir() . 'theme/img/icons/virtual_device.png'
        );

        // add a node for each assigned object type
        $cat_g_id = defined_or_default('C__CATG__SMVIEW_FEED_VIRTUAL_DEVICE');
        $this->_addAssignedObjectTypes($l_tree, $cat_g_id, $node_id, $node_id + 1, 'object_type_mapping_virtual_device');

        $l_template->assign("menu_tree", $l_tree->process($_GET[C__GET__TREE_NODE]));

        $l_posts = isys_module_request::get_instance()->get_posts();
        switch ($l_posts[C__GET__NAVMODE]) {
            case defined_or_default('C__NAVMODE__NEW'):
            case defined_or_default('C__NAVMODE__EDIT'):
                $canEdit = isys_auth_auth::instance()->is_allowed_to(isys_auth::EDIT, "MODULE/C__MODULE__SMVIEW_FEED");
                isys_component_template_navbar::getInstance()
                    ->set_active($canEdit, defined_or_default('C__NAVBAR_BUTTON__SAVE'))
                    ->set_active($canEdit, defined_or_default('C__NAVBAR_BUTTON__CANCEL'));
                break;
        }

        if (isset($_GET['request'])) {
            $instance_id = isset($_GET['instance_id']) ? $_GET['instance_id'] : null;
            $instance_id = isset($_POST['instance_id']) ? $_POST['instance_id'] : $instance_id;
            $this->route($_GET['request'], $instance_id);
        } else {
            $this->route('/');
        }
    }

    /**
     * Get assigned object types of a global category.
     *
     * @param  isys_component_tree $l_tree
     * @param  int $cat_g_id
     * @param  int $parentId
     * @param  int $index
     * @param  string $requestUrl
     * @return int
     * @throws isys_exception_database
     */
    private function _addAssignedObjectTypes($l_tree, $cat_g_id, $parentId, $index, $requestUrl)
    {
        $language = isys_application::instance()->container->get('language');
        $res = $this->_get_assigned_object_types($cat_g_id);
        $icon = $requestUrl == 'object_type_mapping' ? 'jira.png' : 'virtual_device.png';
        while ($row = $res->get_row()) {
            $l_tree->add_node($index, $parentId, $language->get($row['obj_type_title']),
                isys_helper_link::create_url([
                    C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW_FEED'),
                    C__GET__TREE_NODE => $index,
                    'request'         => $requestUrl,
                    'instance_id'     => $row['obj_type_id'],
                ]),
                '',
                self::get_www_dir() . 'theme/img/icons/' . $icon
            );
            $index++;
        }
        return $index;
    }

    /**
     * Routing.
     *
     * @param  string $p_request
     * @param  int|null $p_instance_id
     * @throws isys_exception_database
     * @throws isys_exception_dao
     */
    public function route($p_request, $p_instance_id = null) {
        $l_template = $this->m_tpl;
        $l_posts = isys_module_request::get_instance()->get_posts();
        if (empty($p_instance_id)) {
            $items = isset($l_posts['id']) ? $l_posts['id'] : [];
        } else {
            $items = array($p_instance_id);
        }

        switch ($p_request) {
            case 'adapter_jira':
                $manager = new JiraManager();
                $l_template->include_template('contentbottomcontent', __DIR__ . '/templates/jira_management.tpl');
                $this->_manage($manager, $l_posts, $p_instance_id, $items);
                break;
            case 'adapter_idoit':
                $manager = new IdoitManager();
                $l_template->include_template('contentbottomcontent', __DIR__ . '/templates/idoit_management.tpl');
                $this->_manage($manager, $l_posts, $p_instance_id, $items);
                break;
            case 'adapter_virtual_device':
                $this->_handle_virtual_device();
                break;
            case 'object_type_mapping':
                switch ($l_posts[C__GET__NAVMODE]) {
                    case defined_or_default('C__NAVMODE__SAVE'):
                        $mapping_dao = new isys_cmdb_dao_smview_feed_jira_2_obj_type(isys_application::instance()->database);
                        $filter_dao = new isys_cmdb_dao_smview_feed_jira_filter(isys_application::instance()->database);
                        $this->_save_object_type_config($p_instance_id, $mapping_dao, $filter_dao);
                        break;
                }
                $this->_process_object_type_config_page_jira($p_instance_id);
                $l_template->include_template('contentbottomcontent', __DIR__ . '/templates/mapping.tpl');
                break;
            case 'object_type_mapping_virtual_device':
                switch ($l_posts[C__GET__NAVMODE]) {
                    case defined_or_default('C__NAVMODE__SAVE'):
                        $mapping_dao = new isys_cmdb_dao_smview_feed_virtual_device_2_obj_type(isys_application::instance()->database);
                        $filter_dao = new isys_cmdb_dao_smview_feed_virtual_device_filter(isys_application::instance()->database);
                        $this->_save_object_type_config($p_instance_id, $mapping_dao, $filter_dao);
                        break;
                }
                $this->_process_object_type_config_page_virtual_device($p_instance_id);
                $l_template->include_template('contentbottomcontent', __DIR__ . '/templates/mapping.tpl');
                break;
            default:
                switch ($l_posts[C__GET__NAVMODE]) {
                    case defined_or_default('C__NAVMODE__SAVE'):
                        $this->_save_cache_config();
                        break;
                }
                $this->_process_cache_config_page();
                $l_template->include_template('contentbottomcontent', __DIR__ . '/templates/index.tpl');
                break;
        }
    }

    /**
     * Manage common CRUD operations.
     *
     * @param  AbstractManager $manager
     * @param  array $l_posts
     * @param  int $p_instance_id
     * @param  array $items
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    private function _manage($manager, $l_posts, $p_instance_id, $items) {
        switch ($l_posts[C__GET__NAVMODE]) {
            case defined_or_default('C__NAVMODE__NEW'):
                $manager->handleNewMode();
                $manager->processNewPage();
                break;
            case defined_or_default('C__NAVMODE__SAVE'):
                $manager->saveObject($p_instance_id);
                $manager->processListing();
                break;
            case defined_or_default('C__NAVMODE__DELETE'):
                foreach ($items as $id) {
                    $manager->deleteObject($id);
                }
                $manager->processListing();
                break;
            default:
                if (! empty($items)) {
                    $id = $items[0];
                    $manager->processPage($id);
                } else {
                    $manager->processListing();
                }
        }
    }

    /**
     * Process cache configuration page.
     *
     * @throws isys_exception_database
     */
    private function _process_cache_config_page()
    {
        $dao = new isys_cmdb_dao_smview_feed_configuration(isys_application::instance()->database);
        $type = $dao->get_object('id', 1);
        $l_rules = [];
        foreach ($this->_cache_mapping as $db => $field) {
            $l_rules[$field]['p_strValue'] = $type[$db];
        }
        $l_rules['C__SMVIEW_FEED__ADDRESS']['p_strPlaceholder'] = 'http(s)://192.168.1.2:8000';

        $l_navbar = isys_component_template_navbar::getInstance();
        $canEdit = isys_auth_auth::instance()->is_allowed_to(isys_auth::EDIT, "MODULE/C__MODULE__SMVIEW_FEED");
        if ($_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__EDIT')) {
            $l_navbar->set_selected($canEdit, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
                ->set_active($canEdit, defined_or_default('C__NAVBAR_BUTTON__SAVE'))
                ->set_active($canEdit, defined_or_default('C__NAVBAR_BUTTON__CANCEL'));
        } else {
            $l_navbar->set_active($canEdit, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
                ->set_visible($canEdit, defined_or_default('C__NAVBAR_BUTTON__EDIT'));
        }

        isys_application::instance()->template
            ->smarty_tom_add_rules("tom.content.bottom.content", $l_rules);
    }

    /**
     * Save cache configuration.
     *
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    private function _save_cache_config()
    {
        $dao = new isys_cmdb_dao_smview_feed_configuration(isys_application::instance()->database);
        isys_component_template_navbar::getInstance()->set_active(isys_auth_auth::instance()->is_allowed_to(isys_auth::EDIT, "MODULE/C__MODULE__SMVIEW_FEED"),
            defined_or_default('C__NAVBAR_BUTTON__EDIT'));

        $params = [];
        foreach ($this->_cache_mapping as $db => $field) {
            $value = isset($_POST[$field]) ? $_POST[$field] : null;
            $params[$db] = $value;
        }

        try {
            $type = $dao->get_object('id', 1);
            if (empty($type)) {
                $dao->create_object($params);
            } else {
                $dao->update_object(1, $params);
            }
        } catch (UnexpectedValueException $e) {
            $_POST["navMode"] = defined_or_default('C__NAVMODE__EDIT');
        }

        $l_rules = [];
        foreach ($this->_cache_mapping as $db => $field) {
            $l_rules[$field]['p_strValue'] = $params[$db];
        }
        isys_application::instance()->template->smarty_tom_add_rules("tom.content.bottom.content", $l_rules);
    }

    /**
     * Get assigned object types.
     *
     * @param  int $cat_g_id
     * @return isys_component_dao_result
     * @throws isys_exception_database
     */
    private function _get_assigned_object_types($cat_g_id)
    {
        $dao = new isys_cmdb_dao_category_g_global(isys_application::instance()->database);
        $sql = "SELECT
            l.isys_obj_type_2_isysgui_catg__isys_obj_type__id AS obj_type_id,
            t.isys_obj_type__title AS obj_type_title
            FROM isys_obj_type_2_isysgui_catg l
            JOIN isys_obj_type t ON l.isys_obj_type_2_isysgui_catg__isys_obj_type__id = t.isys_obj_type__id
            WHERE isys_obj_type_2_isysgui_catg__isysgui_catg__id = $cat_g_id";
        return $dao->retrieve($sql);
    }

    /**
     * Process jira configuration page.
     *
     * @param  int $p_instance_id
     * @throws isys_exception_database
     * @throws Exception
     */
    private function _process_object_type_config_page_jira($p_instance_id)
    {
        $language = isys_application::instance()->container->get('language');
        $l_template = isys_application::instance()->template;
        $l_template->assign('obj_type_title', $this->_getObjectTypeName($p_instance_id));

        // get all JIRA fields
        try {
            $proxy_api = new Jira();
            // @todo: determine instance(s) to be queried
            $fields = $proxy_api->get_all_fields();
        } catch (isys_exception_api $e) {
            isys_notify::error($language->get('LC__CMDB__CATG__SMVIEW_FEED__CACHE_BACKEND_UNREACHABLE'));
            $fields = [];
        } catch (Exception $e) {
            $fields = [];
        }

        // get current mappings
        $mapping_dao = new isys_cmdb_dao_smview_feed_jira_2_obj_type($this->m_db);
        $mappings = $mapping_dao->filter_objects(array('obj_type_id' => $p_instance_id));

        // get current filters
        $filter_dao = new isys_cmdb_dao_smview_feed_jira_filter($this->m_db);
        $filters = $filter_dao->filter_objects(array('obj_type_id' => $p_instance_id));

        $js_config = $this->_getJsConfig($p_instance_id, 'C__CATG__SMVIEW_FEED_JIRA');
        $js_config['fields'] = $fields;
        $js_config['mappings'] = $mappings;
        $js_config['filters'] = $filters;
        $l_template->assign('assets_dir', isys_module_smview_feed::get_assets_www_dir())
                   ->assign('js_config', json_encode($js_config));

        $l_navbar = isys_component_template_navbar::getInstance();
        if ($_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__EDIT')) {
            $l_navbar->set_active(true, defined_or_default('C__NAVBAR_BUTTON__SAVE'))
                     ->set_active(true, defined_or_default('C__NAVBAR_BUTTON__CANCEL'));
        } else {
            $l_navbar->set_active(true, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
                     ->set_visible(true, defined_or_default('C__NAVBAR_BUTTON__EDIT'));
        }
    }

    /**
     * Process virtual device configuration page.
     *
     * @param  int $p_instance_id
     * @throws isys_exception_database
     * @throws Exception
     */
    private function _process_object_type_config_page_virtual_device($p_instance_id)
    {
        $l_template = isys_application::instance()->template;
        $l_template->assign('obj_type_title', $this->_getObjectTypeName($p_instance_id));

        // get all virtual device fields
        try {
            $proxy_api = new VirtualDevice();
            $fields = $proxy_api->get_all_fields();
        } catch (Exception $e) {
            $fields = [];
        }

        // get current mappings
        $mapping_dao = new isys_cmdb_dao_smview_feed_virtual_device_2_obj_type($this->m_db);
        $mappings = $mapping_dao->filter_objects(array('obj_type_id' => $p_instance_id));

        // get current filters
        $filter_dao = new isys_cmdb_dao_smview_feed_virtual_device_filter($this->m_db);
        $filters = $filter_dao->filter_objects(array('obj_type_id' => $p_instance_id));

        $js_config = $this->_getJsConfig($p_instance_id, 'C__CATG__SMVIEW_FEED_VIRTUAL_DEVICE');
        $js_config['fields'] = $fields;
        $js_config['mappings'] = $mappings;
        $js_config['filters'] = $filters;
        $l_template
            ->assign('assets_dir', isys_module_smview_feed::get_assets_www_dir())
            ->assign('js_config', json_encode($js_config));

        $l_navbar = isys_component_template_navbar::getInstance();
        if ($_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__EDIT')) {
            $l_navbar->set_active(true, defined_or_default('C__NAVBAR_BUTTON__SAVE'))
                ->set_active(true, defined_or_default('C__NAVBAR_BUTTON__CANCEL'));
        } else {
            $l_navbar->set_active(true, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
                ->set_visible(true, defined_or_default('C__NAVBAR_BUTTON__EDIT'));
        }
    }

    /**
     * Get object type name by its ID.
     *
     * @param  int $obj_type_id
     * @return string
     * @throws isys_exception_database
     * @throws Exception
     */
    private function _getObjectTypeName($obj_type_id)
    {
        $dao = new isys_cmdb_dao_category_g_global($this->m_db);
        $sql = "SELECT t.isys_obj_type__title FROM isys_obj_type t WHERE isys_obj_type__id = $obj_type_id";
        $row = $dao->retrieve($sql)->get_row();
        $language = isys_application::instance()->container->get('language');
        return $language->get($row['isys_obj_type__title']);
    }

    /**
     * Get basic javascript configuration.
     *
     * @param  int $p_instance_id
     * @param  string $cat_const
     * @return array
     * @throws isys_exception_database
     * @throws Exception
     */
    private function _getJsConfig($p_instance_id, $cat_const)
    {
        // get object type information and assigned global category to obj_type
        $dao = new isys_cmdb_dao_category_g_global($this->m_db);
        $sql = "SELECT
                    g.isysgui_catg__id,
                    g.isysgui_catg__type,
                    g.isysgui_catg__title,
                    g.isysgui_catg__const,
                    t.isys_obj_type__title
                    FROM isys_obj_type t
                    JOIN isys_obj_type_2_isysgui_catg l ON l.isys_obj_type_2_isysgui_catg__isys_obj_type__id = t.isys_obj_type__id
                    JOIN isysgui_catg g ON l.isys_obj_type_2_isysgui_catg__isysgui_catg__id = g.isysgui_catg__id
                    WHERE isys_obj_type__id = $p_instance_id";
        $res = $dao->retrieve($sql);
        $language = isys_application::instance()->container->get('language');
        $cat_g_list = [];
        while ($row = $res->get_row()) {
            if ($row['isysgui_catg__const'] == $cat_const) {
                continue;
            }
            $cat_g_list[] = [
                'id'    => $row['isysgui_catg__id'],
                'const' => $row['isysgui_catg__const'],
                'title' => $language->get($row['isysgui_catg__title'])
            ];
        }

        // get object type information and assigned specific category to obj_type
        $sql = "SELECT
                    s2.isysgui_cats__id,
                    s2.isysgui_cats__type,
                    s.isysgui_cats__title AS master_title,
                    s2.isysgui_cats__title,
                    s2.isysgui_cats__const,
                    t.isys_obj_type__title
                    FROM isys_obj_type t
                    JOIN isysgui_cats s ON t.isys_obj_type__isysgui_cats__id = s.isysgui_cats__id
                    LEFT JOIN isysgui_cats s2 ON s.isysgui_cats__id = s2.isysgui_cats__parent
                    WHERE isys_obj_type__id = $p_instance_id";
        $res = $dao->retrieve($sql);
        $cat_s_list = [];
        while ($row = $res->get_row()) {
            if ($row['isysgui_cats__const'] == $cat_const) {
                continue;
            }
            $cat_s_list[] = [
                'id'    => $row['isysgui_cats__id'],
                'const' => $row['isysgui_cats__const'],
                'title' => $language->get($row['master_title']) . ' - ' . $language->get($row['isysgui_cats__title']),
            ];
        }

        return [
            'language'    => $this->_get_js_translations(),
            'cat_g_list'  => $cat_g_list,
            'cat_s_list'  => $cat_s_list,
            'ajax_url'    => isys_helper_link::create_url([
                C__GET__AJAX      => 1,
                C__GET__AJAX_CALL => 'get_category_data',
                'func'            => 'get_properties_by_database',
            ]),
            'image_url'   => isys_helper_link::get_base() . 'images/icons/silk/',
            'session'     => $_COOKIE['PHPSESSID'],
            'is_editable' => isys_glob_is_edit_mode(),
        ];
    }

    /**
     * Get translated strings to use with javascript.
     *
     * @return array
     * @throws Exception
     */
    private function _get_js_translations()
    {
        // language templates
        $language_templates = [
            'LC__CMDB__CATG__SMVIEW_FEED__ADD_ASSIGNMENT',
            'LC__CMDB__CATG__SMVIEW_FEED__FIELD_NAME',
            'LC__CMDB__CATG__SMVIEW_FEED__ADD_FIELD_ASSIGNMENT',
            'LC__CMDB__CATG__SMVIEW_FEED__ADD_FILTER_ASSIGNMENT',
            'LC__CMDB__CATG__SMVIEW_FEED__ADD',
            'LC__CMDB__CATG__SMVIEW_FEED__CATEGORY',
            'LC__CMDB__CATG__SMVIEW_FEED__PROPERTY',
            'LC__CMDB__CATG__SMVIEW_FEED__DELETE_ASSIGNMENT',
            'LC__CMDB__CATG__SMVIEW_FEED__FILTER',
            'LC__CMDB__CATG__SMVIEW_FEED__FILTER_EQUAL',
            'LC__CMDB__CATG__SMVIEW_FEED__FILTER_CONTAINS',
            'LC__CMDB__CATG__SMVIEW_FEED__FILTER_NOT_EQUAL',
            'LC__CMDB__CATG__SMVIEW_FEED__FILTER_NOT_CONTAINS',
        ];

        // translate
        $language = [];
        $lang = isys_application::instance()->container->get('language');
        foreach ($language_templates as $l) {
            $language[$l] = $lang->get($l);
        }
        return $language;
    }

    /**
     * Save cache configuration.
     *
     * @param  int $p_instance_id
     * @param  isys_cmdb_dao_smview_feed_abstract $mapping_dao
     * @param  isys_cmdb_dao_smview_feed_abstract $filter_dao
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    private function _save_object_type_config($p_instance_id, $mapping_dao, $filter_dao)
    {
        if ($mapping_dao) {
            $this->_save_mapping_config($p_instance_id, $mapping_dao);
        }
        if ($filter_dao) {
            $this->_save_filter_config($p_instance_id, $filter_dao);
        }
    }

    /**
     * Save cache configuration.
     *
     * @param  int $p_instance_id
     * @param  isys_cmdb_dao_smview_feed_abstract $mapping_dao
     * @throws isys_exception_database
     * @throws isys_exception_dao
     */
    private function _save_mapping_config($p_instance_id, $mapping_dao)
    {
        $mapping = $mapping_dao->filter_objects(array('obj_type_id' => $p_instance_id));
        foreach ($mapping as $m) {
            $mapping_dao->delete_object($m['id']);
        }

        // get mapping IDs
        $ids = [];
        foreach (array_keys($_POST) as $key) {
            if (stripos($key, 'target_field') === 0) {
                $id = str_replace('target_field_', '', $key);
                $ids[] = $id;
            }
        }

        $dao = isys_cmdb_dao_category_g_global::instance($this->m_db);
        foreach ($ids as $id) {
            $field = $_POST['target_field_' . $id] ?: null;
            $cat_selection = $_POST['cat_selection_' . $id] ?: null;
            $cat_property = $_POST['cat_property_' . $id] ?: null;
            if (empty($field) || empty($cat_selection) || empty($cat_property)) {
                continue;
            }

            $catg_id = null;
            $cats_id = null;
            $sql = "SELECT g.* FROM isysgui_catg g WHERE g.isysgui_catg__const = '$cat_selection'";
            $row = $dao->retrieve($sql)->get_row();
            if ($row) {
                $catg_id = $row['isysgui_catg__id'];

                // check if property name is valid
                $sql = "SELECT *
                    FROM isys_property_2_cat
                    WHERE isys_property_2_cat__isysgui_catg__id = $catg_id
                    AND isys_property_2_cat__prop_key = '$cat_property'";
                $row = $dao->retrieve($sql)->get_row();
                $property_name = $row['isys_property_2_cat__prop_key'];
            } else {
                $sql = "SELECT s.* FROM isysgui_cats s WHERE s.isysgui_cats__const = '$cat_selection'";
                $row = $dao->retrieve($sql)->get_row();
                $cats_id = $row['isysgui_cats__id'];

                // check if property name is valid
                $sql = "SELECT *
                    FROM isys_property_2_cat
                    WHERE isys_property_2_cat__isysgui_cats__id = $cats_id
                    AND isys_property_2_cat__prop_key = '$cat_property'";
                $row = $dao->retrieve($sql)->get_row();
                $property_name = $row['isys_property_2_cat__prop_key'];
            }

            $data = [
                'obj_type_id'   => $p_instance_id,
                'field_id'      => $field,
                'field_name'    => $field,
                'cat_const'     => $cat_selection,
                'property_name' => $property_name,
            ];
            if (! empty($cats_id)) {
                $data['cats_id'] = $cats_id;
            }
            if (! empty($catg_id)) {
                $data['catg_id'] = $catg_id;
                unset($data['cats_id']);
            }
            $mapping_dao->create_object($data);
        }
    }

    /**
     * Save cache configuration.
     *
     * @param  int $p_instance_id
     * @param  isys_cmdb_dao_smview_feed_abstract $filter_dao
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    private function _save_filter_config($p_instance_id, $filter_dao)
    {
        $filters = $filter_dao->filter_objects(array('obj_type_id' => $p_instance_id));
        foreach ($filters as $f) {
            $filter_dao->delete_object($f['id']);
        }

        // get filter IDs
        $ids = [];
        foreach (array_keys($_POST) as $key) {
            if (stripos($key, 'target_filter') === 0) {
                $id = str_replace('target_filter_', '', $key);
                $ids[] = $id;
            }
        }

        foreach ($ids as $id) {
            $field = $_POST['target_filter_' . $id] ?: null;
            $filter_condition = $_POST['filter_condition_' . $id] ?: null;
            $filter_value = $_POST['filter_value_' . $id] ?: null;
            if (empty($field) || empty($filter_condition) || empty($filter_value)) {
                continue;
            }

            $data = [
                'obj_type_id' => $p_instance_id,
                'field_id'    => $field,
                'field_name'  => $field,
                'condition'   => $filter_condition,
                'value'       => $filter_value,
            ];
            $filter_dao->create_object($data);
        }
    }

    /**
     * Handle import of uploaded csv file.
     *
     * @throws isys_exception_api
     * @throws Exception
     */
    private function _handle_virtual_device()
    {
        $virtualDevice = new VirtualDevice();
        $properties = $virtualDevice->get_all_fields();

        $l_template = $this->m_tpl;
        $l_template->include_template('contentbottomcontent', __DIR__ . '/templates/virtual_device_upload.tpl');
        $l_template->assign('encType', 'multipart/form-data');
        $l_template->assign('search_properties', $properties);
        if ($_SERVER['REQUEST_METHOD'] === 'POST') {
            $key = 'C__CATS__FILE_UPLOAD_VIRTUAL_DEVICE';
            if ($_FILES[$key]['size'] > 0) {
                // upload
                $this->_import_virtual_devices($key);
            }

            if (isset($_POST['search'])) {
                // search device
                $property = $_POST['search_property'];
                $value = $_POST['search_virtual_device'];
                $this->_search_virtual_devices($property, $value);
            }
        } else {
            // This is necessary for the file-upload.
            isys_component_template_navbar::getInstance()
                ->set_save_mode('formsubmit')
                ->append_button('LC__CMDB__CATG__SMVIEW_FEED__NAVBAR_BUTTON_UPLOAD','upload', [
                    "icon"       => "icons/silk/page_save.png",
                    "js_onclick" => "document.isys_form.submit();",
                ]);
        }
    }

    /**
     * Import virtual devices.
     *
     * @param  string $key
     * @throws isys_exception_api
     */
    private function _import_virtual_devices($key)
    {
        global $g_dirs;

        if (isset($_POST['reset_cache'])) {
            $virtualDevice = new VirtualDevice();
            $virtualDevice->re_index();
        }

        $l_template = $this->m_tpl;
        try {
            $new_file = $this->_save_uploaded_file($key);
            $importer = new VirtualDeviceImporter();
            $log = $importer->import($g_dirs["temp"] . $new_file);
            $l_template->assign('success', true);
            if (! empty($_POST['show_log'])) {
                $l_template->assign('log', $log);
            }
        } catch (Exception $e) {
            $l_template->assign('error', $e->getMessage());
        }
    }

    /**
     * Search virtual devices.
     *
     * @param  string $property
     * @param  string $value
     * @throws Exception
     */
    private function _search_virtual_devices($property, $value)
    {
        $l_template = $this->m_tpl;
        $language = isys_application::instance()->container->get('language');
        $virtualDevice = new VirtualDevice();
        $conditions = ["$property: *$value*"];
        try {
            $virtual_devices = $virtualDevice->query_devices($conditions);
            $msg = empty($virtual_devices) ? $language->get('LC__CMDB__CATG__SMVIEW_FEED__VIRTUAL_DEVICE_NO_RESULT') : '';
            $l_template->assign('search_message', $msg);
        } catch (isys_exception_api $e) {
            $virtual_devices = [];
            $l_template->assign('search_message', $e->getMessage());
        }

        $dao = new isys_cmdb_dao_category_g_global($this->m_db);
        $devices = [];
        foreach ($virtual_devices as $d) {
            $sql = " AND isys_obj__title=" . $dao->convert_sql_text($d['isys_device_name']);
            $row = $dao->get_data(null, null, $sql, null, defined_or_default('C__RECORD_STATUS__NORMAL'))->get_row();
            if ($row) {
                $d['device_url'] = isys_helper_link::get_base() . isys_helper_link::create_url([
                        C__CMDB__GET__OBJECT => $row['isys_obj__id'],
                        C__CMDB__GET__CATG   => defined_or_default('C__CATG__SMVIEW_FEED_VIRTUAL_DEVICE'),
                    ]);

                $virtual_device_dao = new isys_cmdb_dao_category_g_smview_feed_virtual_device($this->m_db);
                $virtual_device_dao->set_object_id($row['isys_obj__id']);
                $connector = $virtual_device_dao->get_connector_by_virtual_device_port_name($d['isys_port']);
                if ($connector) {
                    $d['port_url'] = isys_helper_link::get_base() . isys_helper_link::create_url([
                            C__CMDB__GET__OBJECT   => $row['isys_obj__id'],
                            C__CMDB__GET__CATG     => defined_or_default('C__CATG__SMVIEW_FEED_VIRTUAL_DEVICE'),
                            C__CMDB__GET__CATLEVEL => $connector['isys_catg_connector_list__id']
                        ]);
                }
            }
            $devices[] = $d;
        }

        $l_template->assign('devices', $devices);
        $l_template->assign('search_property', $property);
        $l_template->assign('search_virtual_device', $value);
    }

    /**
     * Save uploaded file.
     *
     * @param  string $key
     * @return string
     */
    private function _save_uploaded_file($key)
    {
        global $g_dirs;

        $l_new_filename = isys_helper_upload::prepare_filename('virtual_device__' . time() . '__' . $_FILES[$key]['name']);
        try {
            isys_helper_upload::save($_FILES[$key], $l_new_filename, $g_dirs["temp"], 0644);
        } catch (isys_exception_filesystem $e) {
            isys_application::instance()->container->get('notify')->error($e->getMessage() . '<br />' . isys_helper_upload::get_error($key) . '<br />');
        }
        return $l_new_filename;
    }
}
