<?php

/**
 * i-doit
 *
 * Class autoloader.
 *
 * @package    idoit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2019 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_module_smview_feed_autoload extends isys_module_manager_autoload
{
    /**
     * Autoloader
     *
     * @param  string $className
     * @return boolean
     */
    public static function init($className)
    {
        $addOnPath = '/src/classes/modules/smview_feed/';
        $class_loader_paths = array(
            '/api/model/',
            '/api/model/feed/',
            '/cmdb/dao/',
            '/cmdb/dao/global/',
            '/cmdb/dao/list/',
            '/cmdb/ui/global/',
        );

        //dynamic loading of classes
        foreach ($class_loader_paths as $clp) {
            $l_path = $clp . $className . '.class.php';
            if (! file_exists(__DIR__ . $l_path)) {
                continue;
            }

            if (parent::include_file($addOnPath . $l_path)) {
                isys_cache::keyvalue()->ns('autoload')->set($className, $addOnPath . $l_path);

                return true;
            }
        }

        // manager classes, available via main menu
        $manager_classes = [
            'AbstractManager',
            'JiraManager',
            'IdoitManager',
        ];

        $l_classes = [
            'isys_ajax_handler_feed_config'           => '/handler/ajax/isys_ajax_handler_feed_config.class.php',
            'idoit\Module\Feed\VirtualDeviceImporter' => '/src/VirtualDeviceImporter.php',
            'isys_popup_new_jira_ticket'              => 'popups/isys_popup_new_jira_ticket.class.php',
        ];

        foreach ($manager_classes as $cls) {
            $class_path = "idoit\Module\Feed\DependentObjectManager\\" . $cls;
            $file_path =  '/src/DependentObjectManager/' . $cls . '.php';
            $l_classes[$class_path] = $file_path;
        }

        $class_path = "idoit\Module\Feed\DaoListResult";
        $file_path =  '/src/DaoListResult.php';
        $l_classes[$class_path] = $file_path;

        // connector classes
        $manager_classes = [
            'Connectors' => [
                'Jira',
                'OrientDB',
            ],
            'ProxyApi' => [
                'AbstractApi',
                'Jira',
                'Idoit',
                'VirtualDevice',
            ],
            'JiraApi' => [
                'Jira',
            ]
        ];

        foreach ($manager_classes as $dir => $classes) {
            foreach ($classes as $cls) {
                $class_path = "idoit\Module\Feed\\" . $dir . "\\" . $cls;
                $file_path =  '/src/' . $dir . DS . $cls . '.php';
                $l_classes[$class_path] = $file_path;
            }
        }

        // include composer's autoloader
        $vendorDir = dirname(__FILE__) . '/vendor/';
        if (file_exists($vendorDir . 'autoload.php')) {
            include_once($vendorDir . 'autoload.php');
        }
        unset($vendorDir);

        if (isset($l_classes[$className])) {
            if (parent::include_file($addOnPath . $l_classes[$className])) {
                isys_cache::keyvalue()->ns('autoload')->set($className, $addOnPath . $l_classes[$className]);

                return true;
            }
        }

        return false;
    }
}
