<?php

/**
 * i-doit
 *
 * Popup for new JIRA ticket.
 *
 * @package    i-doit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_popup_new_jira_ticket extends isys_component_popup
{

    /**
     * Handles Smarty inclusion.
     *
     * @global  array                   $g_config
     *
     * @param   isys_component_template $p_tplclass (unused)
     * @param   mixed                   $p_params   (unused)
     * @return  string
     */
    public function handle_smarty_include(isys_component_template &$p_tplclass, $p_params)
    {
        // This is never used - the popup will directly be triggered via JS callback.
        return null;
    }

    /**
     * Handles module request.
     *
     * @param   isys_module_request $p_modreq
     * @return  isys_component_template
     */
    public function &handle_module_request(isys_module_request $p_modreq)
    {
        try {
            switch ($_POST['func']) {
                case 'new_jira_ticket':
                    $objId = $_POST['obj_id'];
                    if (empty($objId)) {
                        break;
                    }

                    $url = isys_helper_link::create_url([
                        C__GET__AJAX      => 1,
                        C__GET__AJAX_CALL => 'feed_config',
                        'id'              => $objId,
                        'func'            => 'new_jira_ticket',
                    ]);

                    $dao = new isys_cmdb_dao_category_g_global($this->database);
                    $obj = $dao->get_data(null, $objId)->get_row();

                    // get username
                    $userDao = isys_component_dao_user::instance($this->database);
                    $userId = $userDao->get_current_user_id();
                    $username = $userDao->get_user_title($userId);

                    $this->template->activate_editmode()
                        ->assign('obj_id', $objId)
                        ->assign('obj_title', $obj['isys_obj__title'])
                        ->assign('obj_description', strip_tags($obj['isys_obj__description']))
                        ->assign('user', $username)
                        ->assign('url', $url)
                        ->display(isys_module_smview_feed::get_tpl_dir() . '/popup_new_jira_ticket.tpl');
                    break;
            }
            die;
        } catch (Exception $e) {
            $this->template->assign('error', $e->getMessage());
        }
        return $this->template;
    }
}
