<?php

namespace idoit\Module\Feed;

use isys_component_dao_result;

/**
 * DAO List Result class.
 *
 * @package    i-doit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class DaoListResult extends isys_component_dao_result
{
    /**
     * List of all items.
     *
     * @var array
     */
    private $_items;

    /**
     * Internal pointer.
     *
     * @var int
     */
    private $_pointer;

    /**
     * Constructor.
     *
     * @param array $all_items
     * @param int|null $p_category_data_id
     */
    public function __construct($all_items, $p_category_data_id = null)
    {
        $items = $all_items;
        if (! empty($p_category_data_id) && $p_category_data_id != 'FALSE') {
            // get specific issue
            if ($all_items[$p_category_data_id]) {
                $items = [];
                $items[$p_category_data_id] = $all_items[$p_category_data_id];
            } else {
                $items = [];
            }
        }

        $this->_items = $items;
        $this->_pointer = 0;
    }

    /**
     * Returns a row from a DAO result.
     * The result type is specified by $p_result_type and defaults to a assoc+numeric array as result.
     *
     * @param  integer $p_result_type
     * @return array|null
     */
    public function get_row($p_result_type = IDOIT_C__DAO_RESULT_TYPE_ARRAY)
    {
        $keys = array_keys($this->_items);
        if ($keys[$this->_pointer] === false) {
            return null;
        }

        $key = $keys[$this->_pointer];
        $item = $this->_items[$key];
        $this->_pointer++;
        return $item;
    }

    /**
     * Returns the specified key value from the fetched row.
     *
     * @param  string $p_key
     * @return array|null
     */
    public function get_row_value($p_key)
    {
        $keys = array_keys($this->_items);
        if (! $keys[$this->_pointer]) {
            return null;
        }

        $key = $keys[$this->_pointer];
        $item = $this->_items[$key];
        return isset($item[$key]) ? $item[$key] : null;
    }

    /**
     * Converts current dao result into a single array.
     *
     * @param  integer $p_result_type
     * @return array
     */
    public function __to_array($p_result_type = IDOIT_C__DAO_RESULT_TYPE_ARRAY)
    {
        return $this->get_row();
    }

    /**
     * Count method, called by Countable interface.
     *
     * @return int
     */
    public function count(): int
    {
        return count($this->_items);
    }

    /**
     * Retrieves the number of fields from a query.
     *
     * @return integer
     */
    public function num_fields()
    {
        if ($this->count() == 0) {
            return 0;
        }

        $keys = array_keys($this->_items);
        $key = array_shift($keys);

        $item = $this->_items[$key];
        return count(array_keys($item));
    }

    /**
     * Reset internal pointer.
     *
     * @return boolean
     */
    public function reset_pointer()
    {
        $this->_pointer = 0;
        return true;
    }

    /**
     * Free result (?).
     *
     * @return boolean
     */
    public function free_result()
    {
        return true;
    }
}
