<?php

namespace idoit\Module\Feed\DependentObjectManager;

use Exception;
use idoit\Module\Feed\ProxyApi\Jira;
use isys_application;
use isys_cmdb_dao_smview_feed_jira;
use isys_exception_api;
use isys_exception_database;
use isys_helper_link;
use isys_module_smview_feed;

/**
 * Jira Manager class.
 *
 * @package    i-doit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class JiraManager extends AbstractManager
{

    /**
     * Property mapping.
     *
     * @var array
     */
    protected $_mapping = array(
        'address'          => 'C__CATG__SMVIEW_FEED_URL',
        'name'             => 'C__CATG__SMVIEW_FEED_NAME',
        'username'         => 'C__CATG__SMVIEW_FEED_USERNAME',
        'password'         => 'C__CATG__SMVIEW_FEED_PASSWORD',
        'use_api_directly' => 'C__CATG__SMVIEW_FEED_USE_API_DIRECTLY',
    );

    /**
     * Constructor.
     */
    public function __construct()
    {
        $this->_dao = new isys_cmdb_dao_smview_feed_jira(isys_application::instance()->database);
    }

    /**
     * Process generator listing.
     *
     * @throws isys_exception_database
     * @throws Exception
     */
    public function processListing()
    {
        $lang = isys_application::instance()->container->get('language');
        $headers = [
            "id"          => "ID",
            "address"     => $lang->get('LC__CATG__IP_ADDRESS'),
            "name"        => $lang->get('LC_UNIVERSAL__NAME'),
            "username"    => $lang->get('LC__LOGIN__USERNAME'),
            "last_update" => $lang->get('LC__CMDB__CATG__SMVIEW_FEED__JIRA_LAST_UPDATE'),
        ];

        $jiraConfigs = $this->_dao->get_objects();
        $instances = [];
        foreach ($jiraConfigs as $jiraConfig) {
            try {
                $jira = new Jira();
                $instance = $jira->get_instance($jiraConfig['id']);
                $time = strtotime($instance['update_last_finished']);
                $jiraConfig['last_update'] = date('d.m.Y H:i:s', $time);
            } catch (isys_exception_api $e) {
            } catch (isys_exception_database $e) {
            }
            $instances[] = $jiraConfig;
        }

        $title = $lang->get('LC__CMDB__SMVIEW_FEED__ADAPTERS_JIRA');
        $this->_processListing($title, $headers, $instances);
    }

    /**
     * Process page.
     *
     * @param  int $element_id
     * @param  array $additional_fields
     * @throws isys_exception_database
     */
    public function processPage($element_id, $additional_fields = [])
    {
        try {
            $jira = new Jira();
            $instance = $jira->get_instance($element_id);
            $time = strtotime($instance['update_last_finished']);
            $additional_fields['last_update'] = date('d.m.Y H:i:s', $time);
            $additional_fields['last_error'] = $instance['last_error_message'];
        } catch (isys_exception_api $e) {
        } catch (isys_exception_database $e) {
        }
        $additional_fields['assets_dir'] = isys_module_smview_feed::get_assets_www_dir();
        $additional_fields['spinner_url'] = isys_helper_link::get_base() . 'images/ajax-loading.gif';
        $additional_fields['ajax_url'] = isys_helper_link::create_url([
            C__GET__AJAX      => 1,
            C__GET__AJAX_CALL => 'feed_config',
            'func'            => 'reindex',
            'instance_id'     => $element_id]);
        parent::processPage($element_id, $additional_fields);
    }

    /**
     * Add list data to rules array.
     *
     * @param  array $l_rules
     * @return array
     */
    protected function _add_listing_rules($l_rules)
    {
        $l_rules['C__CATG__SMVIEW_FEED_USE_API_DIRECTLY']['p_arData'] = get_smarty_arr_YES_NO();
        $id = (int) $l_rules['C__CATG__SMVIEW_FEED_USE_API_DIRECTLY']['p_strValue'];
        $l_rules['C__CATG__SMVIEW_FEED_USE_API_DIRECTLY']['p_strSelectedID'] = $id;
        return $l_rules;
    }
}
